/*******************************************************************************
  MPLAB Harmony I2C Functions Header File

  Company:
    Microchip Technology Inc.

  File Name:
    main.h

  Summary:
    Contains prototype for
*******************************************************************************/

// DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
*******************************************************************************/
// DOM-IGNORE-END

#ifndef MAIN_H
#define	MAIN_H

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include <stdint.h>
#include <stdlib.h>
#include <stdbool.h>
#include "i2c_bitbang_isr.h"

// *****************************************************************************
// *****************************************************************************
// Section: Function Prototypes
// *****************************************************************************
// *****************************************************************************

#define Start_I2C_Transaction()        I2C_SW_ISR_TRIGGER = 1;

typedef enum
{
    LED_H0_D3_TOGGLE,
            
    LED_H0_D3_IDLE,
            
    LED_H2_D4_TOGGLE,
            
    LED_H2_D4_IDLE,
            
    LED_H1_D5_TOGGLE,
            
    LED_H1_D5_IDLE

} LED_TOGGLE_STATES;

#define EEPROM_address      0xA0

#define RTCC_address        0xDE

typedef enum
{
    I2C_TRANSACTION_SUCCESS,
            
    I2C_TRANSACTION_IN_PROGRESS,
            
    I2C_ADDRESS_BYTE_NACK,
            
    I2C_DATA_BYTE_NACK
            
} I2C_TRANSACTION_STATUS;


typedef struct
{
    uint16_t                address;    
            
    uint8_t*                wrBuf;
    
    uint8_t*                rdBuf;
    
    uint16_t                wrlen;
    
    uint16_t                rdlen;
    
    I2C_OPERATION_STATUS    i2coperationstatus;
    
    uint16_t                wrcnt;
    
    uint16_t                rdcnt;
    
    uint16_t                client;
    
        
}BUFFER_TRANSFER_QUEUE;

uint8_t QueueLength;

void PERFORMANCE_VARS_INITIALIZE(void);

void APP_LED_Initialize(void);

void APP_LED_STATE(void);

void APP_INITIALIZE(void);

void APP_I2C_STATE(void);

void APP_I2C_Initialize(void);

void APP_I2C_BUFFER_TRANSFER_QUEUE(BUFFER_TRANSFER_QUEUE* MessageQueue);

void APP_I2C_POPULATE_QUEUE_DATA(BUFFER_TRANSFER_QUEUE* BTQ3, uint16_t slavedeviceaddress, uint8_t* i2ctxdata, uint8_t* i2crxdata, uint8_t writelength, uint8_t readlength);


#endif	/* MAIN_H */


/*******************************************************************************
 End of File
*/

